var getId = function(){
    var id = 0;
    return (function(){
        return id++;
    });
}();

function Physics(param, pl)
{
    this.G = param.G;
    this.dt = param.dt;
    this.dt2 = param.dt * param.dt;
    this.accuracy = param.accuracy;
    this.player = pl;

    this.Points = new Array();
    this.Walls = new Array();
    this.Springs = new Array();
    this.Sticks = new Array();
}

Physics.prototype.step = function(){
    this.calculateForces();

// set of equations
    for(var i = 0; i < this.accuracy; ++i)
    {
        this.collision();
    	this.satisfyConstraints();
    }
}

Physics.prototype.log = function(str){
    this.player.log(str);
}


Physics.prototype.warn = function(str){
    this.player.warn(str);
}


Physics.prototype.error = function(str){
    this.player.error(str);
}


Physics.prototype.addPoint = function(mass, radius, coorXY, oldCoorXY, isExpToPot){
    var tmp = new Point(mass, radius, coorXY, oldCoorXY, isExpToPot);
    this.Points.push(tmp);
    
    return tmp;
}

Physics.prototype.addWall = function(vectM, vectO){
    var tmp = new Wall(vectM, vectO);
    this.Walls.push(tmp);
    
    return tmp;
}

Physics.prototype.addStick = function(elem1, elem2){
    var tmp = new Stick(elem1, elem2);
    this.Sticks.push(tmp);
    
    return tmp;
}

Physics.prototype.addSpring = function(elem1, elem2, k){
    var tmp = new Spring(elem1, elem2, k);
    this.Springs.push(tmp);
    
    return tmp;
}

Physics.prototype.collision = collision;
Physics.prototype.calculateForces = calculateForces;
Physics.prototype.satisfyConstraints = satisfyConstraints;

    
function calculateForces()
{      
	var Springs = this.Springs;
    var Points = this.Points;
    var Walls = this.Walls;
    var G = this.G;
    var dt2 = this.dt2;
    var k, l;
    
	for(var key in Points)
    {
        k = Points[key];
      	if (k.isExpToPot)
        {
			for(var ley in Points)
			{
                l = Points[ley];
				if(key > ley && l.isExpToPot)
				{
			   	 	var Fs = minusCoor(k.coor, l.coor);
					var deltaFs = deltaCoor(Fs);
					var a = 100;
					if(deltaFs < 2.5 * a)
					{
						var D = Math.pow(10,5);
						var diff = (12 * D / (a * a)) * (Math.pow(a / deltaFs, 14) - Math.pow(a / deltaFs, 8));
						Fs = timesCoor(Fs, diff * dt2);
						this.player.log("diff = " + diff);
					
						l.coor = minusCoor(l.coor, timesCoor(Fs, l.invmass));
						k.coor = plusCoor(k.coor, timesCoor(Fs, k.invmass));
					}
				}
			}
        }
	}
	
// spring forces
/* 	for(var key in Springs)
 	{
	 	var l = Springs[key];

        var Fs = minusCoor(l.coorE, l.coorB);
      	var deltaFs = deltaCoor(Fs);
	 	var diff = l.k_ * ((l.sep_ / deltaFs) - 1);
      	//var D = 4.477 * 1.6 * Math.pow(10,-19);
      	var a = l.sep_;//0.7416 * Math.pow(10,-10);
      	var diff = /*(12*D/(a*a))**//*Math.pow(10,1)*(Math.pow(a/deltaFs,14)-Math.pow(a/deltaFs,8));
        console.log(l.id);
      	console.warn(diff);
        console.error(deltaFs);
      	
      	//if(deltaFs > l.sep_)
        //{
            Fs = timesCoor(Fs, diff*dt2);
            
            if(l.pB_.type == 'point')
            {
                l.pB_.coor = minusCoor(l.pB_.coor, timesCoor(Fs,l.pB_.invmass));
                l.coorB = l.pB_.coor;
            }
            
            if(l.pE_.type == 'point')
            {
                l.pE_.coor = plusCoor(l.pE_.coor, timesCoor(Fs,l.pE_.invmass));
                l.coorE = l.pE_.coor;
            }
    	//}
 //	}*/
    
// points
    for(var key in Points)
    {
        var l = Points[key];
// gravity
        var newCoor = new Coor(2 * l.coor.x - l.oldCoor.x + G.x * dt2, 2 * l.coor.y - l.oldCoor.y + G.y *  dt2);
        l.oldCoor = l.coor;
        l.coor = newCoor;
    }
 
}

function satisfyConstraints()
{
    var Sticks = this.Sticks;
    var Springs = this.Springs;
    //TODO: chck invmass
// sticks  
    for(var key in Sticks)
    {
        var l = Sticks[key];
    
      	var delta = minusCoor(l.coorE,l.coorB);
        var deltalength = deltaCoor(delta);
        var diff;
      	if(l.pE_.invmass + l.pB_.invmass > 0)
        {
           diff = (deltalength-l.length_)/(deltalength*(l.pE_.invmass+l.pB_.invmass));
    	}else{diff = 0;}
      
      	if (l.pB_.type == 'point')
        {
          l.pB_.coor = plusCoor(l.pB_.coor,timesCoor(delta,diff*l.pB_.invmass));
          l.coorB = l.pB_.coor;
        }
      	if (l.pE_.type == 'point')
        {
          l.pE_.coor = minusCoor(l.pE_.coor,timesCoor(delta,diff*l.pE_.invmass));
          l.coorE = l.pE_.coor;
        }
    }
// springs 
    for(var key in Springs)
    {
        var l = Springs[key];
    
      	var delta = minusCoor(l.coorE,l.coorB);
        var deltalength = deltaCoor(delta);
        var diff;
      	if(l.pE_.invmass + l.pB_.invmass > 0)
        {
           diff = (deltalength-l.sep_)/(deltalength*(l.pE_.invmass+l.pB_.invmass));
    	}else{diff = 0;}
      
		/*if(deltalength > 200)
		{
			l.k = 0;
		}else{*/
			diff *= l.k_/100000;
      	
	      	if (l.pB_.type == 'point')
	        {
	          l.pB_.coor = plusCoor(l.pB_.coor,timesCoor(delta,diff*l.pB_.invmass));
	          l.coorB = l.pB_.coor;
	        }
	      	if (l.pE_.type == 'point')
	        {
	          l.pE_.coor = minusCoor(l.pE_.coor,timesCoor(delta,diff*l.pE_.invmass));
	          l.coorE = l.pE_.coor;
	        }
	    /*}*/
    }
}

function collision()
{
    var Points = this.Points;
    var Walls = this.Walls;
// points x points
    for(var key in Points)
    {
        for (var keyI in Points)
        {
            if (key != keyI)
            {
                var l1 = Points[key];
                var l2 = Points[keyI];
              
                if(length(l1.coor, l2.coor) + 0.5 < l1.radius + l2.radius)
                {
                    var vec = new Vector(l1.coor, l2.coor);
                    var deep = l1.radius + l2.radius - length(l1.coor, l2.coor);
                    var lx = Math.sqrt(deep) * (l1.coor.x - l2.coor.x)/length(l1.coor, l2.coor);
                    var ly = Math.sqrt(deep) * (l1.coor.y - l2.coor.y)/length(l1.coor, l2.coor);
                    
                    l1.coor.x += lx * l1.invmass / (l1.invmass + l2.invmass);
                    l2.coor.x -= lx * l2.invmass / (l1.invmass + l2.invmass);
                    
                    l1.coor.y += ly * l1.invmass / (l1.invmass + l2.invmass);
                    l2.coor.y -= ly * l2.invmass / (l1.invmass + l2.invmass);  
                }
            }
        }
    }
// TODO: walls x points
    for(var key in Points)
    {
        for(var keyW in Walls)
        {
            var p = Points[key];
            var w = Walls[keyW];
            
            
        }
    }

// points x borders
/*   for(var key in Points)
    {
        var p = Points[key];
        
        if (p.coor.x + p.radius > 1200)
        {
            p.coor.x = 1200 - p.radius;
        }
        if (p.coor.x - p.radius < 0)
        {
            p.coor.x = p.radius;
        }
        if (p.coor.y + p.radius > 600)
        {
            p.coor.y = 600 - p.radius;
        }
        if (p.coor.y - p.radius  < 0)
        {
            p.coor.y = p.radius;
        }
    }*/
}

